/******************************************************************************
 *
 * Filename: hashtable.h
 *
 * Author:   Robert Thornburrow
 * Project:  Cycle - The CYbot Control LanguagE
 * Created:  11th June 2003
 *
 * Purpose:  Java-style hashtable for convenience.
 *           Based on implementation from JavaVM.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/


#ifndef _hashtable_h

#define _hashtable_h


#ifdef __cplusplus
extern "C" {
#endif


/**
 * An entry in the linked list of a hashtable bucket.  This is really private,
 * but memory needs it to free the loaded table.
 */
typedef struct _Hashtable_Entry
{
	struct _Hashtable_Entry* next;		 /* Pointer to next entry or NULL */
	const char*				 key;		 /* This entry's key (non-NULL) */
	void*					 value;		 /* This entry's value (non-NULL) */
} Hashtable_Entry;


/**
 * The hashtable itself - entries are hidden, so aren't defined here.
 */
typedef struct _Hashtable
{
	int						  bucketc;  /* The number of buckets in bucketv */
	struct _Hashtable_Entry** bucketv;  /* An array of linked lists */
	int 					  count;    /* The total number of entries */
} Hashtable;

typedef void (*DestructorFnPtr)(void*);

extern Hashtable* Hashtable_new( int buckets );
extern void Hashtable_delete( Hashtable* table, size_t valuesize, int keys, int values, DestructorFnPtr destructor );
extern void Hashtable_removeAll( Hashtable* table, size_t valuesize, int keys, int values, DestructorFnPtr destructor );

extern void* Hashtable_put( Hashtable* table, const char* key, void* value );
extern void* Hashtable_get( Hashtable* table, const char* key );

extern const char* Hashtable_findValue( Hashtable* table, void* value );


#ifdef __cplusplus
}
#endif


#endif /* _hashtable_h */
